@@ -207,10 +207,11 @@ class MetadataAgent(object):
             self._process_monitor, datapath, self.conf, namespace)
 
         veth_name = self._get_veth_name(datapath)
+        self.ovs_idl.del_port(
+            veth_name[0], bridge=self.conf.ovs_integration_bridge).execute()
         if ip_lib.device_exists(veth_name[0]):
             ip_lib.IPWrapper().del_veth(veth_name[0])
 
-        self.ovs_idl.del_port('br-int', veth_name[0]).execute()
         ip.garbage_collect_namespace()
 
     def update_datapath(self, datapath):
@@ -316,7 +317,8 @@ class MetadataAgent(object):
 
         # Configure the OVS port and add external_ids:iface-id so that it
         # can be tracked by OVN.
-        self.ovs_idl.add_port('br-int', veth_name[0]).execute()
+        self.ovs_idl.add_port(self.conf.ovs_integration_bridge,
+                              veth_name[0]).execute()
         self.ovs_idl.db_set(
             'Interface', veth_name[0],
             ('external_ids', {'iface-id': port.logical_port})).execute()
