@@ -13,10 +13,12 @@
 #
 
 from neutron_lib.api.definitions import l3
+from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
+from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
@@ -27,6 +29,7 @@ from oslo_utils import excutils
 from neutron.db import common_db_mixin
 from neutron.db import extraroute_db
 from neutron.db import l3_gwmode_db
+from neutron.extensions import external_net
 
 from networking_ovn.common import extensions
 from networking_ovn.common import ovn_client
@@ -391,18 +394,42 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                               '%(error)s', {'id': fip['id'], 'error': e})
         return router_ids
 
+    def _get_gateway_port_physnet_mapping(self):
+        # This function returns all gateway ports with corresponding
+        # external network's physnet
+        net_physnet_dict = {}
+        port_physnet_dict = {}
+        l3plugin = directory.get_plugin(plugin_constants.L3)
+        if not l3plugin:
+            return port_physnet_dict
+        context = n_context.get_admin_context()
+        for net in l3plugin._plugin.get_networks(
+            context, {external_net.EXTERNAL: [True]}):
+            if net.get(pnet.NETWORK_TYPE) in [n_const.TYPE_FLAT,
+                                              n_const.TYPE_VLAN]:
+                net_physnet_dict[net['id']] = net.get(pnet.PHYSICAL_NETWORK)
+        for port in l3plugin._plugin.get_ports(context, filters={
+            'device_owner': [n_const.DEVICE_OWNER_ROUTER_GW]}):
+            port_physnet_dict[port['id']] = net_physnet_dict.get(
+                port['network_id'])
+        return port_physnet_dict
+
     def schedule_unhosted_gateways(self):
-        valid_chassis_list = self._sb_ovn.get_all_chassis()
+        port_physnet_dict = self._get_gateway_port_physnet_mapping()
+        chassis_physnets = self._sb_ovn.get_chassis_and_physnets()
         unhosted_gateways = self._ovn.get_unhosted_gateways(
-            valid_chassis_list)
-        if unhosted_gateways:
-            with self._ovn.transaction(check_error=True) as txn:
-                for g_name, r_options in unhosted_gateways.items():
-                    chassis = self.scheduler.select(self._ovn, self._sb_ovn,
-                                                    g_name)
-                    r_options['redirect-chassis'] = chassis
-                    txn.add(self._ovn.update_lrouter_port(g_name,
-                                                          options=r_options))
+            port_physnet_dict, chassis_physnets)
+        with self._ovn.transaction(check_error=True) as txn:
+            for g_name, r_options in unhosted_gateways.items():
+                physnet = port_physnet_dict.get(g_name[len('lrp-'):])
+                candidates = [chassis
+                              for chassis, physnets in chassis_physnets.items()
+                              if physnet and physnet in physnets]
+                chassis = self.scheduler.select(
+                    self._ovn, self._sb_ovn, g_name, candidates=candidates)
+                r_options['redirect-chassis'] = chassis
+                txn.add(self._ovn.update_lrouter_port(g_name,
+                                                      options=r_options))
 
     @staticmethod
     @registry.receives(resources.SUBNET, [events.AFTER_UPDATE])
