@@ -10,8 +10,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron.agent.ovsdb.native import commands
-from neutron.agent.ovsdb.native import idlutils
+from ovsdbapp.backend.ovs_idl import command
+from ovsdbapp.backend.ovs_idl import idlutils
 
 from networking_ovn._i18n import _
 from networking_ovn.common import utils
@@ -50,7 +50,7 @@ def get_lsp_dhcp_options_uuids(lsp, lsp_name):
     return uuids
 
 
-class AddLSwitchCommand(commands.BaseCommand):
+class AddLSwitchCommand(command.BaseCommand):
     def __init__(self, api, name, may_exist, **columns):
         super(AddLSwitchCommand, self).__init__(api)
         self.name = name
@@ -67,9 +67,13 @@ class AddLSwitchCommand(commands.BaseCommand):
         row.name = self.name
         for col, val in self.columns.items():
             setattr(row, col, val)
+        self.result = row.uuid
+
+    def post_commit(self, txn):
+        self.result = txn.get_insert_uuid(self.result)
 
 
-class DelLSwitchCommand(commands.BaseCommand):
+class DelLSwitchCommand(command.BaseCommand):
     def __init__(self, api, name, if_exists):
         super(DelLSwitchCommand, self).__init__(api)
         self.name = name
@@ -88,7 +92,7 @@ class DelLSwitchCommand(commands.BaseCommand):
         self.api._tables['Logical_Switch'].rows[lswitch.uuid].delete()
 
 
-class LSwitchSetExternalIdCommand(commands.BaseCommand):
+class LSwitchSetExternalIdCommand(command.BaseCommand):
     def __init__(self, api, name, field, value, if_exists):
         super(LSwitchSetExternalIdCommand, self).__init__(api)
         self.name = name
@@ -114,7 +118,7 @@ class LSwitchSetExternalIdCommand(commands.BaseCommand):
         lswitch.external_ids = external_ids
 
 
-class AddLSwitchPortCommand(commands.BaseCommand):
+class AddLSwitchPortCommand(command.BaseCommand):
     def __init__(self, api, lport, lswitch, may_exist, **columns):
         super(AddLSwitchPortCommand, self).__init__(api)
         self.lport = lport
@@ -152,9 +156,13 @@ class AddLSwitchPortCommand(commands.BaseCommand):
             setattr(port, col, val)
         # add the newly created port to existing lswitch
         _addvalue_to_list(lswitch, 'ports', port.uuid)
+        self.result = port.uuid
+
+    def post_commit(self, txn):
+        self.result = txn.get_insert_uuid(self.result)
 
 
-class SetLSwitchPortCommand(commands.BaseCommand):
+class SetLSwitchPortCommand(command.BaseCommand):
     def __init__(self, api, lport, if_exists, **columns):
         super(SetLSwitchPortCommand, self).__init__(api)
         self.lport = lport
@@ -171,7 +179,7 @@ class SetLSwitchPortCommand(commands.BaseCommand):
             msg = _("Logical Switch Port %s does not exist") % self.lport
             raise RuntimeError(msg)
 
-        # Delete DHCP_Options records no longer refered by this port.
+        # Delete DHCP_Options records no longer referred by this port.
         # The table rows should be consistent for the same transaction.
         # After we get DHCP_Options rows uuids from port dhcpv4_options
         # and dhcpv6_options references, the rows shouldn't disappear for
@@ -179,15 +187,21 @@ class SetLSwitchPortCommand(commands.BaseCommand):
         cur_port_dhcp_opts = get_lsp_dhcp_options_uuids(
             port, self.lport)
         new_port_dhcp_opts = set()
-        dhcpv4_options = self.columns.pop('dhcpv4_options', [])
-        if isinstance(dhcpv4_options, list):
+        dhcpv4_options = self.columns.pop('dhcpv4_options', None)
+        if dhcpv4_options is None:
+            new_port_dhcp_opts.update([option.uuid for option in
+                                       getattr(port, 'dhcpv4_options', [])])
+        elif isinstance(dhcpv4_options, list):
             new_port_dhcp_opts.update(dhcpv4_options)
             port.dhcpv4_options = dhcpv4_options
         else:
             new_port_dhcp_opts.add(dhcpv4_options.result)
             port.dhcpv4_options = [dhcpv4_options.result]
-        dhcpv6_options = self.columns.pop('dhcpv6_options', [])
-        if isinstance(dhcpv6_options, list):
+        dhcpv6_options = self.columns.pop('dhcpv6_options', None)
+        if dhcpv6_options is None:
+            new_port_dhcp_opts.update([option.uuid for option in
+                                       getattr(port, 'dhcpv6_options', [])])
+        elif isinstance(dhcpv6_options, list):
             new_port_dhcp_opts.update(dhcpv6_options)
             port.dhcpv6_options = dhcpv6_options
         else:
@@ -200,7 +214,7 @@ class SetLSwitchPortCommand(commands.BaseCommand):
             setattr(port, col, val)
 
 
-class DelLSwitchPortCommand(commands.BaseCommand):
+class DelLSwitchPortCommand(command.BaseCommand):
     def __init__(self, api, lport, lswitch, if_exists):
         super(DelLSwitchPortCommand, self).__init__(api)
         self.lport = lport
@@ -219,7 +233,7 @@ class DelLSwitchPortCommand(commands.BaseCommand):
             msg = _("Port %s does not exist") % self.lport
             raise RuntimeError(msg)
 
-        # Delete DHCP_Options records no longer refered by this port.
+        # Delete DHCP_Options records no longer referred by this port.
         cur_port_dhcp_opts = get_lsp_dhcp_options_uuids(
             lport, self.lport)
         for uuid in cur_port_dhcp_opts:
@@ -229,7 +243,7 @@ class DelLSwitchPortCommand(commands.BaseCommand):
         self.api._tables['Logical_Switch_Port'].rows[lport.uuid].delete()
 
 
-class AddLRouterCommand(commands.BaseCommand):
+class AddLRouterCommand(command.BaseCommand):
     def __init__(self, api, name, may_exist, **columns):
         super(AddLRouterCommand, self).__init__(api)
         self.name = name
@@ -249,7 +263,7 @@ class AddLRouterCommand(commands.BaseCommand):
             setattr(row, col, val)
 
 
-class UpdateLRouterCommand(commands.BaseCommand):
+class UpdateLRouterCommand(command.BaseCommand):
     def __init__(self, api, name, if_exists, **columns):
         super(UpdateLRouterCommand, self).__init__(api)
         self.name = name
@@ -272,7 +286,7 @@ class UpdateLRouterCommand(commands.BaseCommand):
             return
 
 
-class DelLRouterCommand(commands.BaseCommand):
+class DelLRouterCommand(command.BaseCommand):
     def __init__(self, api, name, if_exists):
         super(DelLRouterCommand, self).__init__(api)
         self.name = name
@@ -291,7 +305,7 @@ class DelLRouterCommand(commands.BaseCommand):
         self.api._tables['Logical_Router'].rows[lrouter.uuid].delete()
 
 
-class AddLRouterPortCommand(commands.BaseCommand):
+class AddLRouterPortCommand(command.BaseCommand):
     def __init__(self, api, name, lrouter, **columns):
         super(AddLRouterPortCommand, self).__init__(api)
         self.name = name
@@ -324,8 +338,8 @@ class AddLRouterPortCommand(commands.BaseCommand):
             _addvalue_to_list(lrouter, 'ports', lrouter_port)
 
 
-class UpdateLRouterPortCommand(commands.BaseCommand):
-    def __init__(self, api, name, lrouter, if_exists, **columns):
+class UpdateLRouterPortCommand(command.BaseCommand):
+    def __init__(self, api, name, if_exists, **columns):
         super(UpdateLRouterPortCommand, self).__init__(api)
         self.name = name
         self.columns = columns
@@ -348,7 +362,7 @@ class UpdateLRouterPortCommand(commands.BaseCommand):
             return
 
 
-class DelLRouterPortCommand(commands.BaseCommand):
+class DelLRouterPortCommand(command.BaseCommand):
     def __init__(self, api, name, lrouter, if_exists):
         super(DelLRouterPortCommand, self).__init__(api)
         self.name = name
@@ -375,17 +389,20 @@ class DelLRouterPortCommand(commands.BaseCommand):
         _delvalue_from_list(lrouter, 'ports', lrouter_port)
 
 
-class SetLRouterPortInLSwitchPortCommand(commands.BaseCommand):
-    def __init__(self, api, lswitch_port, lrouter_port):
+class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
+    def __init__(self, api, lswitch_port, lrouter_port, if_exists):
         super(SetLRouterPortInLSwitchPortCommand, self).__init__(api)
         self.lswitch_port = lswitch_port
         self.lrouter_port = lrouter_port
+        self.if_exists = if_exists
 
     def run_idl(self, txn):
         try:
             port = idlutils.row_by_value(self.api.idl, 'Logical_Switch_Port',
                                          'name', self.lswitch_port)
         except idlutils.RowNotFound:
+            if self.if_exists:
+                return
             msg = _("Logical Switch Port %s does not "
                     "exist") % self.lswitch_port
             raise RuntimeError(msg)
@@ -393,9 +410,10 @@ class SetLRouterPortInLSwitchPortCommand(commands.BaseCommand):
         options = {'router-port': self.lrouter_port}
         setattr(port, 'options', options)
         setattr(port, 'type', 'router')
+        setattr(port, 'addresses', 'router')
 
 
-class AddACLCommand(commands.BaseCommand):
+class AddACLCommand(command.BaseCommand):
     def __init__(self, api, lswitch, lport, **columns):
         super(AddACLCommand, self).__init__(api)
         self.lswitch = lswitch
@@ -417,7 +435,7 @@ class AddACLCommand(commands.BaseCommand):
         _addvalue_to_list(lswitch, 'acls', row.uuid)
 
 
-class DelACLCommand(commands.BaseCommand):
+class DelACLCommand(command.BaseCommand):
     def __init__(self, api, lswitch, lport, if_exists):
         super(DelACLCommand, self).__init__(api)
         self.lswitch = lswitch
@@ -445,7 +463,7 @@ class DelACLCommand(commands.BaseCommand):
         _updatevalues_in_list(lswitch, 'acls', old_values=acls_to_del)
 
 
-class UpdateACLsCommand(commands.BaseCommand):
+class UpdateACLsCommand(command.BaseCommand):
     def __init__(self, api, lswitch_names, port_list, acl_new_values_dict,
                  need_compare=True, is_add_acl=True):
         """This command updates the acl list for the logical switches
@@ -600,7 +618,7 @@ class UpdateACLsCommand(commands.BaseCommand):
                                   old_values=acl_del_objs)
 
 
-class AddStaticRouteCommand(commands.BaseCommand):
+class AddStaticRouteCommand(command.BaseCommand):
     def __init__(self, api, lrouter, **columns):
         super(AddStaticRouteCommand, self).__init__(api)
         self.lrouter = lrouter
@@ -620,7 +638,7 @@ class AddStaticRouteCommand(commands.BaseCommand):
         _addvalue_to_list(lrouter, 'static_routes', row.uuid)
 
 
-class DelStaticRouteCommand(commands.BaseCommand):
+class DelStaticRouteCommand(command.BaseCommand):
     def __init__(self, api, lrouter, ip_prefix, nexthop, if_exists):
         super(DelStaticRouteCommand, self).__init__(api)
         self.lrouter = lrouter
@@ -648,7 +666,7 @@ class DelStaticRouteCommand(commands.BaseCommand):
                 break
 
 
-class AddAddrSetCommand(commands.BaseCommand):
+class AddAddrSetCommand(command.BaseCommand):
     def __init__(self, api, name, may_exist, **columns):
         super(AddAddrSetCommand, self).__init__(api)
         self.name = name
@@ -667,7 +685,7 @@ class AddAddrSetCommand(commands.BaseCommand):
             setattr(row, col, val)
 
 
-class DelAddrSetCommand(commands.BaseCommand):
+class DelAddrSetCommand(command.BaseCommand):
     def __init__(self, api, name, if_exists):
         super(DelAddrSetCommand, self).__init__(api)
         self.name = name
@@ -687,7 +705,7 @@ class DelAddrSetCommand(commands.BaseCommand):
         self.api._tables['Address_Set'].rows[addrset.uuid].delete()
 
 
-class UpdateAddrSetCommand(commands.BaseCommand):
+class UpdateAddrSetCommand(command.BaseCommand):
     def __init__(self, api, name, addrs_add, addrs_remove, if_exists):
         super(UpdateAddrSetCommand, self).__init__(api)
         self.name = name
@@ -712,7 +730,7 @@ class UpdateAddrSetCommand(commands.BaseCommand):
             old_values=self.addrs_remove)
 
 
-class UpdateAddrSetExtIdsCommand(commands.BaseCommand):
+class UpdateAddrSetExtIdsCommand(command.BaseCommand):
     def __init__(self, api, name, external_ids, if_exists):
         super(UpdateAddrSetExtIdsCommand, self).__init__(api)
         self.name = name
@@ -737,7 +755,57 @@ class UpdateAddrSetExtIdsCommand(commands.BaseCommand):
         addrset.external_ids = addrset_external_ids
 
 
-class AddDHCPOptionsCommand(commands.BaseCommand):
+class UpdateChassisExtIdsCommand(command.BaseCommand):
+    def __init__(self, api, name, external_ids, if_exists):
+        super(UpdateChassisExtIdsCommand, self).__init__(api)
+        self.name = name
+        self.external_ids = external_ids
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        try:
+            chassis = idlutils.row_by_value(self.api.idl, 'Chassis',
+                                            'name', self.name)
+        except idlutils.RowNotFound:
+            if self.if_exists:
+                return
+            msg = _("Chassis %s does not exist. "
+                    "Can't update external IDs") % self.name
+            raise RuntimeError(msg)
+
+        chassis.verify('external_ids')
+        chassis_external_ids = getattr(chassis, 'external_ids', {})
+        for ext_id_key, ext_id_value in self.external_ids.items():
+            chassis_external_ids[ext_id_key] = ext_id_value
+        chassis.external_ids = chassis_external_ids
+
+
+class UpdatePortBindingExtIdsCommand(command.BaseCommand):
+    def __init__(self, api, name, external_ids, if_exists):
+        super(UpdatePortBindingExtIdsCommand, self).__init__(api)
+        self.name = name
+        self.external_ids = external_ids
+        self.if_exists = if_exists
+
+    def run_idl(self, txn):
+        try:
+            port = idlutils.row_by_value(self.api.idl, 'Port_Binding',
+                                         'logical_port', self.name)
+        except idlutils.RowNotFound:
+            if self.if_exists:
+                return
+            msg = _("Port %s does not exist. "
+                    "Can't update external IDs") % self.name
+            raise RuntimeError(msg)
+
+        port.verify('external_ids')
+        port_external_ids = getattr(port, 'external_ids', {})
+        for ext_id_key, ext_id_value in self.external_ids.items():
+            port_external_ids[ext_id_key] = ext_id_value
+        port.external_ids = port_external_ids
+
+
+class AddDHCPOptionsCommand(command.BaseCommand):
     def __init__(self, api, subnet_id, port_id=None, may_exists=True,
                  **columns):
         super(AddDHCPOptionsCommand, self).__init__(api)
@@ -774,7 +842,7 @@ class AddDHCPOptionsCommand(commands.BaseCommand):
             self.result = txn.get_insert_uuid(self.result)
 
 
-class DelDHCPOptionsCommand(commands.BaseCommand):
+class DelDHCPOptionsCommand(command.BaseCommand):
     def __init__(self, api, row_uuid, if_exists=True):
         super(DelDHCPOptionsCommand, self).__init__(api)
         self.if_exists = if_exists
@@ -790,7 +858,7 @@ class DelDHCPOptionsCommand(commands.BaseCommand):
         self.api._tables['DHCP_Options'].rows[self.row_uuid].delete()
 
 
-class AddNATRuleInLRouterCommand(commands.BaseCommand):
+class AddNATRuleInLRouterCommand(command.BaseCommand):
     # TODO(chandrav): Add unit tests, bug #1638715.
     def __init__(self, api, lrouter, **columns):
         super(AddNATRuleInLRouterCommand, self).__init__(api)
@@ -815,7 +883,7 @@ class AddNATRuleInLRouterCommand(commands.BaseCommand):
         setattr(lrouter, 'nat', nat)
 
 
-class DeleteNATRuleInLRouterCommand(commands.BaseCommand):
+class DeleteNATRuleInLRouterCommand(command.BaseCommand):
     # TODO(chandrav): Add unit tests, bug #1638715.
     def __init__(self, api, lrouter, type, logical_ip, external_ip,
                  if_exists):
@@ -852,7 +920,7 @@ class DeleteNATRuleInLRouterCommand(commands.BaseCommand):
         setattr(lrouter, 'nat', nats)
 
 
-class SetNATRuleInLRouterCommand(commands.BaseCommand):
+class SetNATRuleInLRouterCommand(command.BaseCommand):
     def __init__(self, api, lrouter, nat_rule_uuid, **columns):
         super(SetNATRuleInLRouterCommand, self).__init__(api)
         self.lrouter = lrouter
@@ -876,7 +944,7 @@ class SetNATRuleInLRouterCommand(commands.BaseCommand):
                 break
 
 
-class AddNatIpToLRPortPeerOptionsCommand(commands.BaseCommand):
+class AddNatIpToLRPortPeerOptionsCommand(command.BaseCommand):
     # TODO(chandrav): Add unit tests, bug #1638715.
     def __init__(self, api, lport, nat_ip):
         super(AddNatIpToLRPortPeerOptionsCommand, self).__init__(api)
@@ -908,7 +976,7 @@ class AddNatIpToLRPortPeerOptionsCommand(commands.BaseCommand):
         lport.options = options
 
 
-class DeleteNatIpFromLRPortPeerOptionsCommand(commands.BaseCommand):
+class DeleteNatIpFromLRPortPeerOptionsCommand(command.BaseCommand):
     # TODO(chandrav): Add unit tests, bug #1638715.
     def __init__(self, api, lport, nat_ip):
         super(DeleteNatIpFromLRPortPeerOptionsCommand, self).__init__(api)
@@ -921,7 +989,7 @@ class DeleteNatIpFromLRPortPeerOptionsCommand(commands.BaseCommand):
                                           'name', self.lport)
 
         except idlutils.RowNotFound:
-            msg = _("Logical Switch Port %s does not exist") % self.port
+            msg = _("Logical Switch Port %s does not exist") % self.lport
             raise RuntimeError(msg)
 
         lport.verify('options')
