@@ -70,9 +70,6 @@ class TestOVNMechanismDriver(test_plugin.Ml2PluginV2TestCase):
         self.nb_ovn = self.mech_driver._nb_ovn
         self.sb_ovn = self.mech_driver._sb_ovn
 
-        self.mech_driver._ovn_client = ovn_client.OVNClient(
-            self.nb_ovn, self.sb_ovn)
-
         self.fake_subnet = fakes.FakeSubnet.create_one_subnet().info()
         self.fake_port_no_sg = fakes.FakePort.create_one_port().info()
 
@@ -409,8 +406,8 @@ class TestOVNMechanismDriver(test_plugin.Ml2PluginV2TestCase):
                     self.assertEqual(
                         tools.UnorderedList(
                             ["22:22:22:22:22:22 2.2.2.2",
-                             port['port']['mac_address'] + ' ' + port_ip
-                             + ' ' + '1.1.1.1']),
+                             port['port']['mac_address'] + ' ' + port_ip +
+                             ' ' + '1.1.1.1']),
                         called_args_dict.get('port_security'))
 
                     old_mac = port['port']['mac_address']
@@ -1261,7 +1258,6 @@ class OVNMechanismDriverTestCase(test_plugin.Ml2PluginV2TestCase):
         self.mech_driver._nb_ovn = nb_ovn
         self.mech_driver._sb_ovn = sb_ovn
         self.mech_driver._insert_port_provisioning_block = mock.Mock()
-        self.mech_driver._ovn_client = ovn_client.OVNClient(nb_ovn, sb_ovn)
 
 
 class TestOVNMechansimDriverBasicGet(test_plugin.TestMl2BasicGet,
@@ -1379,7 +1375,6 @@ class TestOVNMechansimDriverSegment(test_segment.HostSegmentMappingTestCase):
         sb_ovn = fakes.FakeOvsdbSbOvnIdl()
         self.mech_driver._nb_ovn = nb_ovn
         self.mech_driver._sb_ovn = sb_ovn
-        self.mech_driver._ovn_client = ovn_client.OVNClient(nb_ovn, sb_ovn)
 
     def _test_segment_host_mapping(self):
         # Disable the callback to update SegmentHostMapping by default, so
@@ -1451,11 +1446,6 @@ class TestOVNMechansimDriverSegment(test_segment.HostSegmentMappingTestCase):
 @mock.patch.object(n_net, 'get_random_mac', lambda *_: '01:02:03:04:05:06')
 class TestOVNMechansimDriverDHCPOptions(OVNMechanismDriverTestCase):
 
-    def setUp(self):
-        super(TestOVNMechansimDriverDHCPOptions, self).setUp()
-        self.mech_driver._ovn_client = ovn_client.OVNClient(
-            self.mech_driver._nb_ovn, self.mech_driver._sb_ovn)
-
     def _test_get_ovn_dhcp_options_helper(self, subnet, network,
                                           expected_dhcp_options,
                                           service_mac=None,
@@ -1531,6 +1521,30 @@ class TestOVNMechansimDriverDHCPOptions(OVNMechanismDriverTestCase):
         self._test_get_ovn_dhcp_options_helper(subnet, network,
                                                expected_dhcp_options)
 
+    def test_get_ovn_dhcp_options_no_gw_ip_but_metadata_ip(self):
+        subnet = {'id': 'foo-subnet', 'network_id': 'network-id',
+                  'cidr': '10.0.0.0/24',
+                  'ip_version': 4,
+                  'enable_dhcp': True,
+                  'dns_nameservers': [],
+                  'host_routes': [],
+                  'gateway_ip': None}
+        network = {'id': 'network-id', 'mtu': 1400}
+
+        expected_dhcp_options = {
+            'cidr': '10.0.0.0/24',
+            'external_ids': {'subnet_id': 'foo-subnet'},
+            'options': {'server_id': '10.0.0.2',
+                        'server_mac': '01:02:03:04:05:06',
+                        'lease_time': str(12 * 60 * 60),
+                        'mtu': '1400',
+                        'classless_static_route':
+                            '{169.254.169.254/32,10.0.0.2}'}}
+
+        self._test_get_ovn_dhcp_options_helper(subnet, network,
+                                               expected_dhcp_options,
+                                               metadata_port_ip='10.0.0.2')
+
     def test_get_ovn_dhcp_options_ipv6_subnet(self):
         subnet = {'id': 'foo-subnet',
                   'cidr': 'ae70::/24',
@@ -1842,8 +1856,6 @@ class TestOVNMechanismDriverMetadataPort(test_plugin.Ml2PluginV2TestCase):
         self.mech_driver._sb_ovn = fakes.FakeOvsdbSbOvnIdl()
         self.nb_ovn = self.mech_driver._nb_ovn
         self.sb_ovn = self.mech_driver._sb_ovn
-        self.mech_driver._ovn_client = ovn_client.OVNClient(
-            self.nb_ovn, self.sb_ovn)
         ovn_config.cfg.CONF.set_override('ovn_metadata_enabled',
                                          True,
                                          group='ovn')
